/*
 * $Id: smartd.h 783309 2016-05-29 20:43:57Z ib-builder $
 *
 * Copyright (C) 2000 Michael Cornwell <cornwell@acm.org>
 *
 * Copyright (c) 2002-2012, Juniper Networks, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * You should have received a copy of the GNU General Public License
 * (for example COPYING); if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __SMARTD_H__
#define __SMARTD_H__

#include "isc/eventlib.h"
#include "atavendor_db.h"

/*
 * JUNIPER: Redefine logging to avoid conflicts
 * with libjuniper
 */
#define logging smartd_logging


/* smartd version number */
#define VERSION_MAJOR 2
#define VERSION_MINOR 1

/*
 * The name used to to refer to this daemon
 */
#define SMARTD		"smartd"
#define PATH_SMARTD_PIDFILE    _PATH_VARRUN SMARTD ".pid"
#define PATH_VMSMARTD           "/usr/bin/vmsmartd.sh" /* JUNOS */

/* Boolean Values */
#ifndef TRUE
#define TRUE	0x01
#define FALSE	0x00
#endif

#define MAXATADEVICES	3

#define SMARTD_CHECKTIME	1800		/* default, 30 minutes */	

#define SMARTD_DISK_INACT_TMR   (10*60)         /* standby timer 10 mins */

/* SMARTD_SLEEPTIME should be less than 60s and 60 should be divisible by it */
#define SMARTD_SLEEPTIME	5		/* seconds */
/* If disk is not in standby then we check every 15s to see if it has gone
 * to standby on its own */
#define SMARTD_POWER_CHECK_1	(SMARTD_SLEEPTIME * 3) /* seconds */
#define SMARTD_POWER_CHECK_OFFSET	5		/* seconds */
/* If disk is in standby then we check every 2s to see if it has come out
 * of standby */
#define SMARTD_POWER_CHECK_2	2		/* seconds */
/* If adaptive standby has been armed then we aggressively check to
 * see if a standby was issued */
#define SMARTD_POWER_CHECK_3	1		/* seconds */
/* If periodic standby is not currently enabled then we wake up every
 * 15s to see if it has been recently enabled and should be used
 */
#define SMARTD_CONFIG_APPLY_TIME        (SMARTD_SLEEPTIME * 3)
/* If disk is in stanby then minimum amount of time we do not disturb
 * it to issue smart checks */
#define SMARTD_STANDBYTIME_MIN	300		/* seconds, 5 minutes */
/* Smartd accumulates and then resets standby times on a daily basis */
#define SMARTD_DAILY_CHECK      (24*60*60)      /* 1 day, in seconds */

#define SMARTD_STANDBY_INTERVAL_DEFAULT	239	   /* 4  hours (in minutes) */
#define SMARTD_STANDBY_INTERVAL_MAX	(30*24*60) /* 30 days (in minutes) */
#define SMARTD_STANDBY_INTERVAL_OFFSET	(15*60)	   /* 15 minutes */

/* Amount by which the summary period should be more than periodic standby
 * interval period
 */
#define SMARTD_STANDBY_SUMMARY_OFFSET   60         /* seconds */

/* If an ioctl failed due to EAGAIN then we retry every 5 seconds upto
 * upto 120 times i.e. a total of 120 minutes
 */
#define SMARTD_IOCTL_RETRY_TIME      5       /* seconds */
#define SMARTD_IOCTL_RETRY_COUNT     120     /* seconds */

/* how often SMART is checks in seconds */
extern unsigned int checktime;
extern unsigned int standbyloglevel;

/* JUNOS begin */

#define SMARTD_CONFIG_FILE      "/etc/smartd_working.conf"    /* JUNOS */
#define SMARTD_MASTER_CONFIG    "/etc/smartd.conf"            /* JUNOS */
#define SMARTD_DEFAULT_DEVICE   "DEFAULT_SSD"
#define KERN_ON_BARE_METAL      "none"                        /* JUNOS */ 
#define KERN_TYPE_SIZE           12                           /* JUNOS */
/* maximum line length in configuration file */
#define MAXLINELEN 1024
/* maximum length of a continued line in configuration file */
#define MAXCONTLINE  4096

extern unsigned char smartd_sleep_infinitely;                 /* JUNOS */

enum parse_flag_enum {
    FOUND_DEVICE_NAME      = (1 << 0), /* device name found in config file  */
    FOUND_NUM_OF_IDS       = (1 << 1), /* num of smart ids to be monitored  */
    FOUND_ATTRIBUTE_IDS    = (1 << 2), /* smart attributes ids monitored    */
    FOUND_THRESHOLD_LIMITS = (1 << 3), /* smart threshold limits for ids    */
    FOUND_VALUE            = (1 << 4), /* value to check raw or norm        */
    FOUND_FLAG             = (1 << 5), /* whether check for low or high     */
    FOUND_END              = (1 << 6)  /* END bit should be the highest bit */
};

/* macro to help parse function check all the flags are parsed */
#define PARSED_ALL_OPTIONS (FOUND_END - 1) /* All Bits of the parse flag set */
#define PARSED_ID  (FOUND_DEVICE_NAME | FOUND_NUM_OF_IDS | FOUND_ATTRIBUTE_IDS)

/* structure to keep track of parse status */
struct idconfig_s {
    unsigned char id;
    enum parse_flag_enum id_status;
};

typedef struct parseinfo_s {
    unsigned char num_of_ids;
    unsigned char no_of_ids_parsed;
    struct idconfig_s config_ids[NUMBER_ATA_SMART_ATTRIBUTES];
    unsigned char no_of_thres_parsed;
    unsigned char no_of_val_parsed;
    unsigned char no_of_flag_parsed;
    enum parse_flag_enum parse_status;
} parseinfo_t;

/* macros for parseinfo */
#define SET_PARSED_STATUS(status)    (parse_info.parse_status |= status)
#define GET_PARSED_STATUS(status)    (parse_info.parse_status & status)
#define CONFIG_ID(IDPARSED)           parse_info.config_ids[IDPARSED].id
#define NO_OF_CONFIG_IDS              parse_info.num_of_ids
#define IDS_PARSED                    parse_info.no_of_ids_parsed
#define THRES_PARSED                  parse_info.no_of_thres_parsed
#define VAL_PARSED                    parse_info.no_of_val_parsed
#define FLAG_PARSED                   parse_info.no_of_flag_parsed
#define FIND_MISSING_OPTIONS(id) \
    (id == 0)  ? (parse_info.parse_status ^ PARSED_ALL_OPTIONS) :   \
    (parse_info.config_ids[id-1].id_status ^ PARSED_ALL_OPTIONS)

/* macros for parsing each configured ids */
#define SET_ID_STATUS(id, status) \
        (parse_info.config_ids[id].id_status |= status)
#define GET_ID_STATUS(id, status) \
        (parse_info.config_ids[id].id_status & status)

/* emum to str */
typedef struct parse_flag_name_s {
    enum parse_flag_enum flag;
    const char *name;
    unsigned char option;
} parse_flag_name_t;
/* JUNOS end */

typedef struct atadevices_s {
	int fd;
	char devicename[14];
	int selftest;
	int npassed;
	struct ata_params drive;
	struct ata_smart_values smartval;
	struct ata_smart_thresholds smartthres;
	struct smart_ssd_thresholds ssdthres;
        /* Index into atasbytab array */
	uint     sby_table_index;
        char    in_standby;     /* Whether disk is in standby */

#define	SBY_SRC_NOTSET		0x0
#define	SBY_SRC_DISK		0x1
#define	SBY_SRC_SMARTD		0x2
        char    sby_source;

	time_t  sby_interval;   /* Interval at which to issue standby */
	time_t  sby_curr_start_time; /* Time at which current standby started */
	time_t  last_power_check_time; /* Last time a power check was done */
	time_t  sby_curr_accum_time;/* Total time disk was in current standby*/
        /* Total time disk was in standby in 24 hrs */
	time_t  sby_daily_accum_time; 
        /* Total time disk was in standby in 24 hrs where each standby was 
         * greater than the threshold sby_adaptive_time_per_sby_min specified
         * in atasbytab */
	time_t  sby_valid_daily_accum_time;
        /* The max continuous standby seen in any one standby in last 24 hrs */
        time_t  sby_continuous_time_max;
        /* Smartd check was skipped once as disk was in standby */
	char    skipped_smart_once;
        /* Kernel has been asked to buffer writes and issue standby */
	char    sby_armed;
        /* Total number of standbys issued for the day */
        uint    sby_daily_num;
	evTimerID power_evtid;
        evTimerID standby_evtid;
} atadevices_t;

enum {
    SBY_NOTNEEDED,      /* Periodic standby should never be enabled */
    SBY_PERIODIC_ALWAYS,
    SBY_PERIODIC_IF_ENABLED, /* Periodic standby only if interval set in CLI */ 
    SBY_ADAPTIVE_ALWAYS,
    SBY_ADAPTIVE_IF_ENABLED, /* Periodic standby only if interval set in CLI */ 
};

typedef struct sby_table_s {
    const char	*disk_model;

#define	IS_24x7		TRUE
#define	NON_24x7	FALSE
    char	is_24x7;
    char	sby_type;

#define	DISABLE_WCACHE	TRUE
#define	ENABLE_WCACHE	FALSE
    char	disable_wcache;
    uint	head_loadunload_cnt_max;
    time_t	accum_time_max;
    time_t	accum_time_per_sby_min;

    /* Stop when accumulated standby's have reached required daily maximum */
#define NOSTOP_AT_LIMIT         0 /* Do not stop issuing standby */
#define STOP_AT_ACCUM_LIMIT     1 /* Stop when accumulated 300s are hit */
#define STOP_AT_CONT_MAX_LIMIT  2 /* Stop when max 300s standby is hit */
    char        stop_at_limit;

    /* Disk's own inactivity timer. If this is zero then disk's
     * inactivity timer is turned off for good. If this is non zero
     * then it is turned off only when smartd is issuing its own
     * adaptive standby */
    unsigned int  inactivity_timer;	/* seconds */
} sby_table_t;

extern uint standbyloglevel;

/*
 * Function Prototypes
 */
extern int daemonloop (char *device);
extern int ataprintmain (int fd);
extern char *atagetsmartattribname (unsigned char id);
extern void logging (int severity, const char *format, ...) __printflike(2, 3);
extern void readstandbyconfig (void);
extern void readconfigfile(void);  /* JUNOS */

/*
 * Function Prototypes
 */
extern int daemonloop (char *device);
extern int ataprintmain (int fd);
extern char *atagetsmartattribname (unsigned char id);
extern void logging (int severity, const char *format, ...);

#endif /* __SMARTD_H__ */
